@@ -9,6 +9,4 @@ let () =
   Pkg.describe "uuidm" @@ fun c ->
   let cmdliner = Conf.value c cmdliner in
   Ok [ Pkg.mllib "src/uuidm.mllib";
-       Pkg.test "test/test";
-       Pkg.bin ~cond:cmdliner "test/uuidtrip";
-       Pkg.test "test/perf"; ]
+       Pkg.bin ~cond:cmdliner "test/uuidtrip" ]
